import { BelongsToUs, clients } from '@crosslab/booking-service-common';
import dayjs from 'dayjs';
import lodash from 'lodash';
import * as mysql from 'mysql2/promise';
import { hackURLWithPort } from '@crosslab/booking-service-common';
import { config } from '../config.js';
import { timetableAnd, timetableNot } from '../timetable.js';
BigInt.prototype.toJSON = function () {
    return this.toString();
};
// TODO: Missing availability since it is not yet well defined
export const postSchedule = async (request, body) => {
    await request.authorization.check_authorization_or_fail('create', `booking`);
    const laterReq = new Map(); // Device in request, device list, request
    let timetables = []; // Booked: Device in request, device list, actual reserved time slots
    let availability = []; // Device in request, device list
    let realDevices = []; // Device in request, device list
    if (!body)
        body = { Experiment: { Devices: [] }, Time: { Start: '', End: '' } };
    // Collect all timetables
    for (let device = 0; device < body.Experiment.Devices.length; device++) {
        // Resolve device
        realDevices.push([]);
        timetables.push([]);
        availability.push([]);
        let r;
        try {
            r = await clients.device.getDevice(body.Experiment.Devices[device].ID, {
                flat_group: true,
            });
        }
        catch (error) {
            console.error('Error while getting device ' +
                body.Experiment.Devices[device].ID +
                ' :', error);
            const err = error;
            // Bad status code
            if (err.response !== undefined && err.response.status !== undefined) {
                if (err.response.status === 503) {
                    return {
                        status: 503,
                    };
                }
                if (err.response.status === 404) {
                    return { status: 404, body: body.Experiment.Devices[device].ID };
                }
                //@ts-ignore: fallback which only works when API can handle errors - still keep it (defensive programming)
                return {
                    status: 500,
                    body: 'Device request ' +
                        body.Experiment.Devices[device].ID +
                        ' returned status code' +
                        err.response.status,
                };
            }
            // any other error
            throw err;
        }
        if (r.url != body.Experiment.Devices[device].ID) {
            return {
                status: 500,
                body: 'API returned bad result (requested device' +
                    body.Experiment.Devices[device].ID +
                    ', got ' +
                    r.url +
                    ')',
            };
        }
        if (r.type === 'device' || r.type === 'cloud instantiable' || r.type === 'edge instantiable') {
            realDevices[device].push(r.url);
        }
        else {
            // group
            for (let i = 0; i < r.devices.length; i++) {
                realDevices[device].push(r.devices[i].url);
            }
        }
        // Process devices
        for (let i = 0; i < realDevices[device].length; i++) {
            timetables[device].push([]);
            // Get timetable
            let d = new URL(realDevices[device][i]);
            let t = [];
            if (!BelongsToUs(d)) {
                // This is not our device
                if (body.onlyOwn) {
                    // This device should not be in the request, jump out
                    return {
                        status: 400,
                    };
                }
                if (laterReq.get(d.origin) === undefined) {
                    let req = lodash.cloneDeep(body);
                    req.onlyOwn = true;
                    req.Experiment.Devices = [];
                    req.Combined = false;
                    laterReq.set(d.origin, [[], [], req]);
                }
                let lr = laterReq.get(d.origin);
                lr[0].push(device);
                lr[1].push(i);
                lr[2].Experiment.Devices.push({ ID: realDevices[device][i] });
                laterReq.set(d.origin, lr);
                t = [];
            }
            else {
                // This is our device
                t = await getTimetables(d, body.Time.Start, body.Time.End);
            }
            timetables[device][i] = t;
            // Get availability
            availability[device].push(clients.device.getDevice(realDevices[device][i], { flat_group: true }));
        }
    }
    const lrpromise = new Map(); // Device in request, device list, request
    for (let k of laterReq.keys()) {
        let lr = laterReq.get(k);
        let req = clients.booking.schedule.schedule(lr[2], { url: k });
        lrpromise.set(k, [lr[0], lr[1], lr[2], req]);
    }
    for (let k of laterReq.keys()) {
        let lr = lrpromise.get(k);
        let req;
        try {
            req = await lr[3];
        }
        catch (error) {
            console.error('Error while getting schedule for ' + k + ' :', error);
            const err = error;
            if (err.response !== undefined && err.response.status !== undefined) {
                if (err.response.status == 503) {
                    return { status: 503 };
                }
                if (err.response.status == 404) {
                    return { status: 404, body: JSON.stringify(err.response.body) };
                }
                return {
                    status: 500,
                    body: 'Institution ' + k + ' returned status code ' + err.response.status,
                };
            }
            throw err;
        }
        if (req.length != lr[2].Experiment.Devices.length) {
            return {
                status: 500,
                body: 'Institution ' +
                    k +
                    ' returned bad result (requested ' +
                    lr[2].Experiment.Devices.length +
                    ' devices, got ' +
                    req.length +
                    ')',
            };
        }
        for (let i = 0; i < req.length; i++) {
            if (req[i].Device != lr[2].Experiment.Devices[i].ID) {
                return {
                    status: 500,
                    body: 'Institution ' +
                        k +
                        ' returned bad result (requested device' +
                        lr[2].Experiment.Devices[i].ID +
                        ', got ' +
                        req[i].Device +
                        ')',
                };
            }
            timetables[lr[i][0]][lr[i][1]] = req[i].Booked;
        }
    }
    let response = [];
    // Build Response
    for (let device = 0; device < body.Experiment.Devices.length; device++) {
        // Get free times
        let free = [];
        for (let i = 0; i < timetables[device].length; i++) {
            // Add availability
            let a;
            try {
                a = await availability[device][i];
            }
            catch (error) {
                console.error('Error while availability for ' +
                    device +
                    ' ' +
                    i +
                    ' :', error);
                const err = error;
                if (err.response !== undefined && err.response.status !== undefined) {
                    // TODO: Remove later if errors are well specified
                    if (err.response.status === 503) {
                        return {
                            status: 503,
                        };
                    }
                    if (err.response.status === 404) {
                        return { status: 404, body: realDevices[device][i] };
                    }
                    //@ts-ignore: fallback which only works when API can handle errors - still keep it (defensive programming)
                    return {
                        status: 500,
                        body: 'Device request ' +
                            realDevices[device][i] +
                            ' returned status code' +
                            err.response.status,
                    };
                }
                throw err;
            }
            if (a.type == 'group') {
                return {
                    status: 500,
                    body: 'Device request ' + realDevices[device][i] + " is group but shouldn't",
                };
            }
            let available;
            if (a.type == 'cloud instantiable' || a.type == 'edge instantiable') {
                available = [{ Start: body.Time.Start, End: body.Time.End }];
            }
            else {
                const announcedAvailability = await clients.device.getDeviceAvailability(a.url, {
                    startTime: body.Time.Start,
                    endTime: body.Time.End,
                });
                available = timetableAnd(announcedAvailability.map(e => {
                    return { Start: e.start, End: e.end };
                }));
            }
            let notAvailable = timetableNot(available, dayjs(body.Time.Start), dayjs(body.Time.End));
            let notFree = timetableAnd(notAvailable, timetables[device][i]);
            console.error(JSON.stringify(timetableNot(notFree, dayjs(body.Time.Start), dayjs(body.Time.End))));
            // Now push free
            free.push(timetableNot(notFree, dayjs(body.Time.Start), dayjs(body.Time.End)));
        }
        let freeCombined = timetableAnd(...free);
        response.push({
            Device: hackURLWithPort(body.Experiment.Devices[device].ID),
            Booked: timetableNot(freeCombined, dayjs(body.Time.Start), dayjs(body.Time.End)),
            Free: freeCombined,
        });
    }
    // Combine if wished
    if (body.Combined) {
        let free = [];
        for (let i = 0; i < response.length; i++) {
            free.push(timetableNot(response[i].Booked, dayjs(body.Time.Start), dayjs(body.Time.End)));
        }
        let freeCombined = timetableAnd(...free);
        response = [
            {
                Device: '<combined>',
                Booked: timetableNot(freeCombined, dayjs(body.Time.Start), dayjs(body.Time.End)),
                Free: freeCombined,
            },
        ];
    }
    return { status: 200, body: response };
};
export async function getTimetables(device, start, end) {
    let r = [];
    let db = await mysql.createConnection(config.ReservationDSN);
    try {
        await db.connect();
        let [rows, _fields] = await db.execute('SELECT start, end FROM reservation WHERE `device`=? AND ((`start` < ? AND `end` > ?) OR (`start` < ? AND `end` > ?) OR (`start` > ? AND `end` < ?) OR (`start` < ? AND `end` > ?)) ORDER BY `start` ASC', [
            device.toString(),
            dayjs(start).toDate(),
            dayjs(start).toDate(),
            dayjs(end).toDate(),
            dayjs(end).toDate(),
            dayjs(start).toDate(),
            dayjs(end).toDate(),
            dayjs(start).toDate(),
            dayjs(end).toDate(),
        ]);
        for (let i = 0; i < rows.length; i++) {
            r.push({
                Start: dayjs(rows[i].start).toISOString(),
                End: dayjs(rows[i].end).toISOString(),
            });
        }
    }
    finally {
        db.end();
    }
    // flatten r
    r = timetableNot(r, dayjs(start), dayjs(end));
    r = timetableNot(r, dayjs(start), dayjs(end));
    return r;
}
export default { postSchedule };
//# sourceMappingURL=index.js.map